import pymongo
import network_constants as netconst
from database.MongoRepositoryBase import MongoRepositoryBase
import json

from db.agi.agi_repository import AgiRepository

from db.entities import Location, UserCluster, PopularLocation
from typing import List


class Repository(MongoRepositoryBase):

    def __init__(self, agi_data=False):
        super().__init__(netconst.COMMUNITY_DETECTION_DB_HOSTNAME, 
            netconst.COMMUNITY_DETECTION_DB_PORT, 'communityDetectionDb')

        self._location_collection = 'location_agi' if agi_data else 'location'
        self._location_cluster_collection = 'location_cluster_agi' if agi_data else 'location_cluster'
        self._time_cluster_collection = 'time_cluster_agi' if agi_data else 'time_cluster'

        self.agi_repo = AgiRepository()

    def add_location(self, location: Location):
        super().insert_entry(self._location_collection, location.to_serializable_dict())

    def get_locations(self) -> List[Location]:
        locations = super().get_entries(self._location_collection)
        return [Location(l) for l in locations]

    def get_agi_locations(self) -> List[Location]:
        agi_locations = self.agi_repo.getLocations()
        return [Location(agi_loc) for agi_loc in agi_locations]

    def add_location_cluster(self, cluster: UserCluster):
        super().insert_entry(self._location_cluster_collection, cluster.to_serializable_dict(for_db=True))

    def get_location_clusters(self) -> List[UserCluster]:
        clusters = super().get_entries(self._location_cluster_collection)
        return [UserCluster(c['cluster_label'], json.loads(c['clusters'])) for c in clusters]

    def add_time_cluster(self, cluster: UserCluster):
        super().insert_entry(self._time_cluster_collection, cluster.to_serializable_dict(for_db=True))

    def get_time_clusters(self) -> List[UserCluster]:
        clusters = super().get_entries(self._time_cluster_collection)
        return [UserCluster(c['cluster_label'], json.loads(c['clusters'])) for c in clusters]

    def add_popular_location(self, popular_location: PopularLocation):
        pass
